/** Result of the `generate` function. */
type GenerateResult = {
    /** Successful generation. */
    success: true;
    /** Generated ID from the `generate` function. */
    result: string;
} | {
    /** Failed generation. */
    success: false;
    /** Error from the `generate` function. */
    error: Error;
};

/** Result of the `verify` function. */
type VerifyResult = {
    /** Successful verification. */
    success: true;
    /** Generated date of the ID verified by the `verify` function. */
    result: Date;
    /** Whether the ID was generated naturally. */
    natural: boolean;
} | {
    /** Failed verification. */
    success: false;
    /** Error from the `verify` function. */
    error: Error;
};

/** Options for the `RowIDWithConfig` function. */
type RowIDWithConfigOptions = {
    /**
     * The list of characters that can be used in the RowID,
     * it must be longer or equal to `28`.
     * @default "0123456789ABCDEFGHJKMNPQRSTVWXYZ"
     */
    charList?: string;
    /**
     * The default length of randomness in the RowID,
     * it's recommended to be longer or equal to `6`.
     * @default 22
     */
    randomnessLength?: number;
};
/**
 * This function allows you to customize how RowID works,
 * and returns the modified functions based on the parameters.
 *
 * ### Example
 *
 * ```typescript
 * import type { RowIDWithConfigResult } from "rowid";
 * import { RowIDWithConfig } from "rowid";
 *
 * const {
 *     RowID,
 *     encode,
 *     decode,
 *     generate,
 *     verify,
 *     getRandomness,
 * }: RowIDWithConfigResult = RowIDWithConfig({
 *     // ...
 * });
 * ```
 */
declare const RowIDWithConfig: (options: RowIDWithConfigOptions) => {
    /**
     * This function generates a unique ID
     * that is almost impossible to duplicate.
     *
     * Or you can specify the number of randomness,
     * a larger number will generate a longer ID,
     * with less chance of collision.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { RowID }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const id: string = RowID();
     * ```
     */
    RowID: (randomnessLength?: number) => string;
    /**
     * This function encodes the timestamp into a ID without randomness.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { encode }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const encoded: string = encode(new Date().getTime());
     * ```
     */
    encode: (timestamp: number) => string;
    /**
     * This function decodes the ID into a Date.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { decode }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const decoded: Date = decode("0123456789");
     * ```
     */
    decode: (encoded: string) => Date;
    /**
     * This function generates a ID based on the input.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult, GenerateResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { generate }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const current: number = new Date().getTime();
     * const result: GenerateResult = generate(current, 32 - 10);
     * ```
     */
    generate: (timestamp: number, randomnessLength?: number) => GenerateResult;
    /**
     * This function verifies if the ID is valid and natural.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult, VerifyResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { verify }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const result: VerifyResult = verify("0123456789");
     * ```
     */
    verify: (encoded: string) => VerifyResult;
    /**
     * This function generates randomness with different methods based on the environment.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { getRandomness }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const result: string = getRandomness(6);
     * ```
     */
    getRandomness: (randomnessLength: number) => string;
};
/** Result of the `RowIDWithConfig` function. */
type RowIDWithConfigResult = ReturnType<typeof RowIDWithConfig>;

/**
 * This function generates a 32-character unique ID
 * that is almost impossible to duplicate.
 *
 * Or you can specify the number of randomness,
 * a larger number will generate a longer ID,
 * with less chance of collision.
 *
 * ### Example
 *
 * ```typescript
 * import RowID from "rowid";
 *
 * const id: string = RowID();
 * ```
 *
 * With specified randomness:
 *
 * ```typescript
 * import RowID from "rowid";
 *
 * const id: string = RowID(6);
 * ```
 */
declare const RowID: (randomnessLength?: number) => string;
/**
 * This function encodes the timestamp into a ID without randomness.
 *
 * ### Example
 *
 * ```typescript
 * import { encode } from "rowid";
 *
 * const result: string = encode(new Date().getTime());
 * ```
 */
declare const encode: (timestamp: number) => string;
/**
 * This function decodes the ID into a Date.
 *
 * ### Example
 *
 * ```typescript
 * import { RowID, decode } from "rowid";
 *
 * const id: string = RowID();
 * const result: Date = decode(id);
 * ```
 */
declare const decode: (encoded: string) => Date;
/**
 * This function generates a ID based on the input.
 *
 * ### Example
 *
 * ```typescript
 * import type { GenerateResult } from "rowid";
 * import { generate } from "rowid";
 *
 * const current: number = new Date().getTime();
 * const result: GenerateResult = generate(current, 22);
 * ```
 */
declare const generate: (timestamp: number, randomnessLength?: number) => GenerateResult;
/**
 * This function verifies if the ID is valid and natural.
 *
 * ### Example
 *
 * ```typescript
 * import type { VerifyResult } from "rowid";
 * import { RowID, verify } from "rowid";
 *
 * const id: string = RowID();
 * const result: VerifyResult = verify(id);
 * ```
 */
declare const verify: (encoded: string) => VerifyResult;
/**
 * This function generates randomness with different methods based on the environment.
 *
 * ### Example
 *
 * ```typescript
 * import { getRandomness } from "rowid";
 *
 * const result: string = getRandomness(6);
 * ```
 */
declare const getRandomness: (randomnessLength: number) => string;

export { type GenerateResult, RowID, RowIDWithConfig, type RowIDWithConfigOptions, type RowIDWithConfigResult, type VerifyResult, decode, RowID as default, encode, generate, getRandomness, verify };
